"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const GQLField_1 = require("./GQLField");
const TypeParser_1 = require("../../helpers/TypeParser");
const Arg_1 = require("./Arg");
const ArrayHelper_1 = require("../../helpers/ArrayHelper");
const Middleware_1 = require("../middlewares/Middleware");
const Args_1 = require("./Args");
class Field extends GQLField_1.GQLField {
    constructor() {
        super(...arguments);
        this._args = [];
        this._middlewares = [];
    }
    get args() {
        return this._args;
    }
    get flatArgs() {
        return this.args.flatMap((a) => {
            if (a instanceof Args_1.Args) {
                return a.args;
            }
            return a;
        });
    }
    get resolve() {
        return this._resolve;
    }
    static create(name, type) {
        return new Field(name, type);
    }
    setMiddlewares(...middlewares) {
        this._middlewares = middlewares;
        return this;
    }
    addMiddlewares(...middlewares) {
        return this.setMiddlewares(...this._middlewares, ...middlewares);
    }
    removeMiddlewares(...middlewares) {
        return this.setMiddlewares(...ArrayHelper_1.ArrayHelper.remove(middlewares, this._middlewares));
    }
    setArgs(...args) {
        this._args = args.filter((a) => a);
        return this;
    }
    addArg(nameOrArg, type) {
        if (typeof nameOrArg === "string") {
            this.args.push(Arg_1.Arg.create(nameOrArg, type));
        }
        else {
            this.args.push(nameOrArg);
        }
        return this;
    }
    addArgs(...args) {
        return this.setArgs(...this.args, ...args);
    }
    removeArgs(...args) {
        return this.setArgs(...ArrayHelper_1.ArrayHelper.remove(args, this._args));
    }
    setResolver(resolver, ...args) {
        const instance = new resolver();
        this._resolve = instance.resolve.bind(instance);
        if (args.length > 0) {
            this.setArgs(...args);
        }
        else {
            this.setArgs(instance.getArgs());
        }
        return this;
    }
    setResolve(resolve, ...args) {
        this.setArgs(...args);
        this._resolve = resolve;
        return this;
    }
    build() {
        const args = this._args.flatMap((arg) => arg.build());
        this.addMiddlewares(Middleware_1.Middleware.create(this.resolve, "__main"));
        const field = {
            name: this._name,
            type: TypeParser_1.TypeParser.parse(this._type),
            resolve: this.resolveField,
            deprecationReason: this._deprecationReason || null,
            isDeprecated: !!this._deprecationReason,
            args,
            description: this._description,
            extensions: [],
        };
        this._built = field;
        return { ...this._built };
    }
    async resolveField(source, args, context, infos) {
        const next = async (args, gql, index, paramsToNext) => {
            const nextFn = () => next(args, gql, index + 1, paramsToNext);
            const guardToExecute = this._middlewares[index].function;
            const res = await guardToExecute(args, gql, nextFn, paramsToNext);
            if (res) {
                return res;
            }
            return paramsToNext;
        };
        return await next(args, {
            source,
            context,
            infos,
        }, 0, {});
    }
}
exports.Field = Field;
//# sourceMappingURL=Field.js.map